%%%% PLEASE UNCOMMENT TO RUN
clear
close all
%% TestCases to RUN!!!

%      testcase = 'Plot Convergence'
%      testcase = 'Plot Invariants Preservation'
     testcase = 'Plot Solution'

%% Initial Values
m0 = [cos(1.1);0;sin(1.1)];   %Initial value of M
I = diag([2;1;2/3]);  %Inertia's matrix
ax = [1,0,0];       %Axis of rotation
angle=pi/4;         %angle of rotation
q0 = [ cos(angle/2) , sin(angle/2)*ax]';
tol = 1e-10; % Tolerance
N_h = 5; 
error_IM = zeros(1,N_h);
error_KM = zeros(1,N_h);
error_SM = zeros(1,N_h);
error_RK4 = zeros(1,N_h);
error_GL4 = zeros(1,N_h);
load('y_exact.mat')% Exact Solution

%% PLOTS
switch testcase
%% Convergence Plots
    case 'Plot Convergence'
            for N=1:N_h
                % IM
                y_IM = IM(1,N,m0,q0,I,tol);
                % KM
                y_KM = KM(1,N,m0,q0,I,tol);
                % SM
                y_SM = SM(1,N,m0,q0,I,tol);
                % RK4
                y_RK4 = RK4(1,N,m0,q0,I);
                % GL4
                y_GL4 = GL4(1,N,m0,q0,I,tol);
                %%Error Compuation 
                error_IM(N+1)=norm( y_IM(:,end) - savedY(end,2:end)');
                error_KM(N+1)=norm( y_KM(:,end) - savedY(end,2:end)');
                error_SM(N+1)=norm( y_SM(:,end) - savedY(end,2:end)');
                error_RK4(N+1)=norm( y_RK4(:,end) - savedY(end,2:end)');
                error_GL4(N+1)=norm( y_GL4(:,end) - savedY(end,2:end)');
            end        
                    stp=1:N_h+1;
                    h = 1./2.^stp;
                    loglog(h,error_IM,'-+','LineWidth',1.5,'MarkerSize',10)
                    hold on
                    loglog(h,error_KM,'-s','LineWidth',1.5,'MarkerSize',10)
                    loglog(h,error_SM,'-p','LineWidth',1.5,'MarkerSize',10)
                    loglog(h,error_RK4,'-o','LineWidth',1.5,'MarkerSize',10)
                    loglog(h,error_GL4,'-d','LineWidth',1.5,'MarkerSize',10)
                    axis equal
                    title('Convergence Plots')
                    xlabel('Step-Size')
                    ylabel('Global Error')
                    set(gca, 'FontName', 'Times New Roman','FontSize',11)
                    p_IM = polyfit(log10(h(2:end)),log10(error_IM(2:end)),1);
                    p_KM = polyfit(log10(h(2:end)),log10(error_KM(2:end)),1);
                    p_SM = polyfit(log10(h(2:end)),log10(error_SM(2:end)),1);
                    p_RK4 = polyfit(log10(h(2:end)),log10(error_RK4(2:end)),1);
                    p_GL4 = polyfit(log10(h(2:end)),log10(error_GL4(2:end)),1);
                    Scheme = {'IM';'KM';'SM';'RK4';'GL4'};
                    ConvergenceRate = [p_IM(1);p_KM(1);p_SM(1);p_RK4(1);p_GL4(1)];
                    T = table(Scheme,ConvergenceRate)
                    legend('IM','KM','SM','RK4','GL4')
                    
%% Invariants Plots
    case 'Plot Invariants Preservation' 
                N = 11; T = 1000;
                h=T/2^N;
                % IM
                y_IM = IM(T,N,m0,q0,I,tol);
                % KM
                y_KM = KM(T,N,m0,q0,I,tol);
                % SM
                y_SM = SM(T,N,m0,q0,I,tol);
                % RK4
                y_RK4 = RK4(T,N,m0,q0,I);
                % GL4
                y_GL4 = GL4(T,N,m0,q0,I,tol);
                %%InvariantChecks
                invar_IM = abs(sum((y_IM(1:3,:)').^2,2) - sum((y_IM(1:3,1)').^2,2));
                invar_KM = abs(sum((y_KM(1:3,:)').^2,2) - sum((y_IM(1:3,1)').^2,2));
                invar_SM = abs(sum((y_SM(1:3,:)').^2,2) - sum((y_IM(1:3,1)').^2,2));
                invar_RK4 = abs(sum((y_RK4(1:3,:)').^2,2) - sum((y_IM(1:3,1)').^2,2));
                invar_GL4 = abs(sum((y_GL4(1:3,:)').^2,2) - sum((y_IM(1:3,1)').^2,2));
                energy_IM = abs(.5*sum((y_IM(1:3,:)').*(I\y_IM(1:3,:))',2) - .5*sum((y_IM(1:3,1)').*(I\y_IM(1:3,1))',2));
                energy_KM = abs(.5*sum((y_KM(1:3,:)').*(I\y_KM(1:3,:))',2) - .5*sum((y_IM(1:3,1)').*(I\y_IM(1:3,1))',2));
                energy_SM = abs(.5*sum((y_SM(1:3,:)').*(I\y_SM(1:3,:))',2) - .5*sum((y_IM(1:3,1)').*(I\y_IM(1:3,1))',2));
                energy_RK4 = abs(.5*sum((y_RK4(1:3,:)').*(I\y_RK4(1:3,:))',2) - .5*sum((y_IM(1:3,1)').*(I\y_IM(1:3,1))',2));
                energy_GL4 = abs(.5*sum((y_GL4(1:3,:)').*(I\y_GL4(1:3,:))',2) - .5*sum((y_IM(1:3,1)').*(I\y_IM(1:3,1))',2));
                timesteps = 0:h:T;
                figure
                semilogy(timesteps,invar_IM,'-+','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:100:length(y_IM))
                hold on
                semilogy(timesteps,invar_KM,'-s','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:100:length(y_IM))
                semilogy(timesteps,invar_SM,'-p','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:100:length(y_IM))
                semilogy(timesteps,invar_RK4,'-o','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:100:length(y_IM))
                semilogy(timesteps,invar_GL4,'-d','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:100:length(y_IM))
                title('Error in Invariant P^TP Preservation')
                xlabel('Time')
                ylabel('Error')
                legend('IM','KM','SM','RK4','GL4','Location','east')
                set(gca, 'FontName', 'Times New Roman','FontSize',11)
                figure
                semilogy(timesteps,energy_IM,'-+','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:100:length(y_IM))
                hold on
                title('Error in Energy Preservation')
                semilogy(timesteps,energy_KM,'-s','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:100:length(y_IM))
                semilogy(timesteps,energy_SM,'-p','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:100:length(y_IM))
                semilogy(timesteps,energy_RK4,'-o','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:100:length(y_IM))
                semilogy(timesteps,energy_GL4,'-d','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:100:length(y_IM))
                legend('IM','KM','SM','RK4','GL4','Location','east')
                xlabel('Time')
                ylabel('Error')
                set(gca, 'FontName', 'Times New Roman','FontSize',11)
    case 'Plot Solution'
                N = 13; T = 1000;
                h=T/2^N;
                % IM
                y_GL4 = GL4(T,N,m0,q0,I,tol);
                plot3(y_GL4(1,:),y_GL4(2,:),y_GL4(3,:),'LineWidth',1.5,'MarkerSize',10)
                grid on
                xlabel('$M_1$','Interpreter','latex')
                ylabel('$M_2$','Interpreter','latex')
                zlabel('$M_3$','Interpreter','latex')
                set(gca, 'FontName', 'Times New Roman','FontSize',11)
end



