function [F]  = Error_shootRM(vxv0)
global N phi epsilon L;

q0 = eye(3);
QL = [cos(phi), -sin(phi), 0;
      sin(phi), cos(phi), 0;
           0   ,   0     , 1];
q0 = rotMToEulerP(q0);
qL = rotMToEulerP(QL);      

Cm = [110.8, 0, 0;
      0, 110.8, 0;
      0,0 , 85.2];
Cp = [2338.6, 0, 0;
      0, 2338.6, 0;
      0,0 , 6080.5];
 
h = L/(N);
v_i = 0:h:L;
n = length(v_i);

x0 = zeros(3,1);
xL = [0;0;(1-epsilon)*L];

vx0 = vxv0(1:3);
v0 = [0;vxv0(5:7)];

tol = 1e-10;
z = zeros(14,1);
z(:,1) =[x0;q0;vx0;v0]; 

    for j=1:n-1
        xn = z(1:3,j); qn = z(4:7,j); vxn = z(8:10,j); vn = z(11:14,j);
        e = 1;
        qk0 = zeros(4,1);
        vhalfk0 = [0;zeros(3,1)];
        iter = 1;
        lambdak0 = 1e-5*ones(4,1);
        while e>tol
            vxn1 = vxn;
            vhalfk1 = vn - h/2*(partialSq(vxn1,vhalfk0,qn,Cm,Cp) + 2*qn.*lambdak0);
            xk1 = xn + h/2*(partialSvx(vxn1,qn,Cp) + partialSvx(vxn1,qk0,Cp));
            qk1 = qn + h/2*(partialSv(vhalfk0,qn,Cm) + partialSv(vhalfk0,qk0,Cm));
                vhalfk0 = vhalfk1; qk0 = qk1; 
            Evhalf = vhalfk1 -( vn - h/2*(partialSq(vxn1,vhalfk0,qn,Cm,Cp) + 2*qn.*lambdak0));
            Ex = xk1 -( xn + h/2*(partialSvx(vxn1,qn,Cp) + partialSvx(vxn1,qk0,Cp)));
            Eq = qk1 -( qn + h/2*(partialSv(vhalfk0,qn,Cm) + partialSv(vhalfk0,qk0,Cm)));
            e = norm([Evhalf;Ex;Eq]);
            iter = iter + 1;
            if iter ==100
                fprintf('Max iterations reached: Error:%e\n',e)
                break
            end
        end
        vhalf = vhalfk1; qn1 = qk1; xn1 = xk1;
            qn1 = qn1/norm(qn1);                       
            %% Finding mu
        mun1 = 1/(h*dot(qn1,qn1))*(dot(qn1,vhalf) - h/2*dot(qn1,partialSq(vxn1,vhalf,qn1,Cm,Cp)));       
        %% Projecting back the solution
        vn1 = vhalf - h/2*(partialSq(vxn1,vhalf,qn1,Cm,Cp) + 2*qn1.*mun1);
        Eqv = dot(qn1,vn1);
        %% Last Constraint
            
        z(:,j+1) = [xn1;qn1;vxn1;vn1];      
    end
%     y_ref = z;
   F = [z(1:3,end);z(4:7,end)] - [xL;qL];
   nrm = norm(F);
        save('sol2','z','v_i','nrm')
end

%% Functions
% Partial S/ partial x
function PSx = partialSx()
    PSx = zeros(3,1);
end
% Partial S/ partial q
function PSq = partialSq(vx,v,q,Cm,Cn)
    Q = QuatToMat(q);
    vxQuat = [0;vx];
    [Lvx,Rvx] = matQuat(vxQuat);
    [Lv,Rv] = matQuat(v);
    [Lq,Rq] = matQuat(q);
    qc = [q(1);-q(2:4)];
    [Lqc,Rqc] = matQuat(qc); 
    Cninv = diag(1./diag(Cn));
    Dninv = Q*Cninv*Q';
    CmTilda = [1e5 , zeros(1,3);
               zeros(3,1) , Cm];
    CmTildainv = CmTilda\eye(4);
    DmTildainv = Rqc*Lq*CmTildainv*Rq*Lqc;
    e3 = [0;0;1];
    PSq = .25*Rq*Lv*Lqc*DmTildainv*Rqc*v +...
          Rq*(Lvx - Rvx)*[0;Dninv*vx - Q*e3] +...
          2*dot((Q-eye(3))*vx,Dninv*vx - Q*e3)*q;
end
% Partial S/ partial vx
function PSvx = partialSvx(vx,q,Cn)
        e3 = [0;0;1];
        Q = QuatToMat(q);
        Cninv = diag(1./diag(Cn));
        Dninv = Q*Cninv*Q';       
        PSvx = (-Dninv*vx + Q*e3);
end
% Partial S/ partial v
function PSv = partialSv(v,q,Cm)
        [Lq,Rq] = matQuat(q);
        qc = [q(1);-q(2:4)];
        [Lqc,Rqc] = matQuat(qc);
        CmTilda = [1e5 , zeros(1,3);
                    zeros(3,1) , Cm];
        CmTildainv = CmTilda\eye(4);
        DmTildainv = Rqc*Lq*CmTildainv*Rq*Lqc;
        PSv = -.25*Rq*DmTildainv*Rqc*v;
end
function vhat = hat(v);
    vhat = [0, -v(3) , v(2);
            v(3) , 0 , -v(1);
            -v(2), v(1) , 0];        
end
function [L,R] = matQuat(q)
    L = [q(1) , -q(2:4)';
         q(2:4) , q(1)*eye(3) + hat(q(2:4))];
    R = [q(1) , -q(2:4)';
         q(2:4) , q(1)*eye(3) - hat(q(2:4))];
end
function [R]=QuatToMat(Quat)
    R = [(1-2*(Quat(3)^2 + Quat(4)^2))         ,(-2*Quat(1)*Quat(4)+2*Quat(2)*Quat(3))  , (2*Quat(1)*Quat(3)+2*Quat(2)*Quat(4));
         (2*Quat(1)*Quat(4)+2*Quat(2)*Quat(3)) , (1 - 2*(Quat(2)^2 + Quat(4)^2))        , (-2*Quat(1)*Quat(2)+2*Quat(3)*Quat(4));
        (-2*Quat(1)*Quat(3)+2*Quat(2)*Quat(4)) , (2*Quat(1)*Quat(2)+2*Quat(3)*Quat(4))  , (1 - 2*(Quat(2)^2 + Quat(3)^2)) ];
end