function [F]  = Error_shootSE(vxv0)
global N phi epsilon L;

q0 = eye(3);
QL = [cos(phi), -sin(phi), 0;
      sin(phi), cos(phi), 0;
           0   ,   0     , 1];
q0 = rotMToEulerP(q0);
qL = rotMToEulerP(QL);      

Cm = [110.8, 0, 0;
      0, 110.8, 0;
      0,0 , 85.2];
Cn = [2338.6, 0, 0;
      0, 2338.6, 0;
      0,0 , 6080.5];
 
h = L/(N);
v_i = 0:h:L;
n = length(v_i);

x0 = zeros(3,1);
xL = [0;0;(1-epsilon)*L];

vx0 = vxv0(1:3);
v0 = [0;vxv0(5:7)];

tol = 1e-10;
z = zeros(14,1);
z(:,1) =[x0;q0;vx0;v0]; 

    for j=1:n-1
        xn = z(1:3,j); qn = z(4:7,j); vxn = z(8:10,j); vn = z(11:14,j);
        e = 1.;
        vxn0 = zeros(3,1);
        vn0 = zeros(4,1);
        iter = 1;
        while e>tol
            clear vxn1 vn1 xn1 qn1
            vxn1 = vxn - h*partialSx();
            vn1  = vn  - h*partialSq(vxn0,vn0,qn,Cm,Cn);
            xn1 = xn + h*partialSvx(vxn0,qn,Cn);
            qn1 = qn + h*partialSv(vn0,qn,Cm);
                vxn0 = vxn1;
                vn0  = vn1;
            Ev = vn1  - vn  + h*partialSq(vxn0,vn0,qn,Cm,Cn);
            Ex = xn1 - xn - h*partialSvx(vxn0,qn,Cn);
            Eq = qn1 - qn - h*partialSv(vn0,qn,Cm);
            Error = [Ev;Ex;Eq];
            e = norm(Error);
            iter = iter + 1;
            if iter ==100
                fprintf('Max iterations reached: Error:%e\n',e)
                break
            end
        end
            qn1 = qn1/norm(qn1);
        z(:,j+1) = [xn1;qn1;vxn1;vn1]; 
          
    end
   F = [z(1:3,end);z(4:7,end)] - [xL;qL];
   nrm = norm(F);
        save('sol2','z','v_i','nrm')
end

%% Functions
% Partial S/ partial x
function PSx = partialSx()
    PSx = zeros(3,1);
end
% Partial S/ partial q
function PSq = partialSq(vx,v,q,Cm,Cn)
    Q = QuatToMat(q);
    vxQuat = [0;vx];
    [Lvx,Rvx] = matQuat(vxQuat);
    [Lv,Rv] = matQuat(v);
    [Lq,Rq] = matQuat(q);
    qc = [q(1);-q(2:4)];
    [Lqc,Rqc] = matQuat(qc); 
    Cninv = diag(1./diag(Cn));
    Dninv = Q*Cninv*Q';
    CmTilda = [1e5 , zeros(1,3);
               zeros(3,1) , Cm];
    CmTildainv = CmTilda\eye(4);

    DmTildainv = Rqc*Lq*CmTildainv*Rq*Lqc;
    e3 = [0;0;1];
    PSq = (1/4*Rq*Lv*Lqc*DmTildainv*Rqc*v +...
          Rq*(Lvx - Rvx)*[0;Dninv*vx - Q*e3]); %changed here
end
% Partial S/ partial vx
function PSvx = partialSvx(vx,q,Cn) 
        e3 = [0;0;1];
        Q = QuatToMat(q);
        Cninv = diag(1./diag(Cn));
        Dninv = Q*Cninv*Q';       
        PSvx = -Dninv*vx + Q*e3;
end
% Partial S/ partial v
function PSv = partialSv(v,q,Cm)
        [Lq,Rq] = matQuat(q);
        qc = [q(1);-q(2:4)];
        [Lqc,Rqc] = matQuat(qc);
        CmTilda = [1e5 , zeros(1,3);
                    zeros(3,1) , Cm];
        CmTildainv = CmTilda\eye(4);
        DmTildainv = Rqc*Lq*CmTildainv*Rq*Lqc;
        PSv = -1/4*Rq*DmTildainv*Rqc*v;
end
function vhat = hat(v);
    vhat = [0, -v(3) , v(2);
            v(3) , 0 , -v(1);
            -v(2), v(1) , 0];        
end
function [L,R] = matQuat(q)
    L = [q(1) , -q(2:4)';
         q(2:4) , q(1)*eye(3) + hat(q(2:4))];
    R = [q(1) , -q(2:4)';
         q(2:4) , q(1)*eye(3) - hat(q(2:4))];
end
function [R]=QuatToMat(Quat)
    R = [(1-2*(Quat(3)^2 + Quat(4)^2))         ,(-2*Quat(1)*Quat(4)+2*Quat(2)*Quat(3))  , (2*Quat(1)*Quat(3)+2*Quat(2)*Quat(4));
         (2*Quat(1)*Quat(4)+2*Quat(2)*Quat(3)) , (1 - 2*(Quat(2)^2 + Quat(4)^2))        , (-2*Quat(1)*Quat(2)+2*Quat(3)*Quat(4));
        (-2*Quat(1)*Quat(3)+2*Quat(2)*Quat(4)) , (2*Quat(1)*Quat(2)+2*Quat(3)*Quat(4))  , (1 - 2*(Quat(2)^2 + Quat(3)^2)) ];
end