%%%%% PLEASE UNCOMMENT TO RUN
clear 
close all
%% TEST CASES!!!
%  testcase = 'Plot Convergence'
%  testcase = 'Plot Solution'
%% Initial Values 
global N phi epsilon L;

N_h = 9;
vx0    = [.5;0;.5];
M0 = [cos(1.1);0;sin(1.1)]; 
v0 = [0;M0];
z0 = [vx0;v0];
tol = 1e-10;
options = optimoptions('fsolve','Display','iter','Algorithm','levenberg-marquardt','PlotFcn',[],...
            'FunctionTolerance',tol);
switch testcase
    case 'Plot Solution'
         phiVal = 0:pi/2:pi;
         L= 10;
         epsilon = 0.4;
         for k=1:length(phiVal)
             phi = phiVal(k); 
             N = 512;
             %SE
%             [x_SE,F_SE] = fsolve(@Error_shootSE,z0,options);
%             acr_SE = norm(F_SE);
%             load('sol2.mat')
%             z_SE = z;
%             clear z
            %RM
            [x_RM,F_RM] = fsolve(@Error_shootRM,z0,options);
            acr_RM = norm(F_RM);
            load('sol2.mat')
            z_RM = z;
            clear z       
            rodplot(z_RM) % Plot the Solution
            hold on
         end
    case 'Plot Convergence'
        load('RefSol.mat')
        L= 1;
        epsilon = 0.5;
        phi = pi;
        h_ref = 1/2048;
        v_ref = 0:h_ref:1;
        
        for k = 4:N_h
             N = 2^k;
             %SE
             tic
            [x_SE,F_SE] = fsolve(@Error_shootSE,z0,options);
            t_SE(k-3) = toc;
            acr_SE = norm(F_SE);
            load('sol2.mat')
            z_SE = z;
            clear z
            %RM
            tic
            [x_RM,F_RM] = fsolve(@Error_shootRM,z0,options);
            t_RM(k-3) = toc;
            acr_RM = norm(F_RM);
            load('sol2.mat')
            z_RM = z;
            clear z           
            %%% Computing the Error
            a_i = find(ismember(v_i,v_ref));
            b_ref = find(ismember(v_ref,v_i));
            %%Computing Error
            for ept =1 : length(a_i)
                error_SE(k-3) = norm(y_ref(:,b_ref(ept)) - z_SE(:,a_i(ept)),Inf);
                error_RM(k-3) = norm(y_ref(:,b_ref(ept)) - z_RM(:,a_i(ept)),Inf);
            end
        end
        %% 
        fsolveAccuracy = [acr_SE;acr_RM];
        stp = 4:N_h;
        H = 1./2.^stp;
        figure
        loglog(H,error_SE,'-s','LineWidth',1.5,'MarkerSize',10)
        hold on
        loglog(H,error_RM,'-o','LineWidth',1.5,'MarkerSize',10)
        %%% Computing Order of Convergence
        p_SE = polyfit(log10(H),log10(error_SE),1);
        p_RM = polyfit(log10(H),log10(error_RM),1);
        xlabel('Step-size')
        ylabel('Global Error')
        set(gca, 'FontName', 'Times New Roman')
        box on       
        ConvergenceRate = [p_SE(1);p_RM(1)];
        Scheme = {'SE';'RM'};
        T = table(Scheme,ConvergenceRate,fsolveAccuracy)
        legend('SE','RM','Location','northwest')
end 
 %% Functions
function rodplot(z)
plot3(-z(1,:),-z(2,:),z(3,:),'*','MarkerSize',5)    
xlabel('x')
ylabel('y')
zlabel('z')
set(gca, 'FontName', 'Times New Roman')
grid on
box on
end
