function [F]  = Error_shootRK4(mp0)
global N phi epsilon L Cm Cp;

q0 = eye(3);
QL = [cos(phi), -sin(phi), 0;
      sin(phi), cos(phi), 0;
           0   ,   0     , 1];
q0 = rotMToEulerP(q0);
qL = rotMToEulerP(QL);      
h = L/(N);
v_i = 0:h:L;
n = length(v_i);

x = zeros(3,1);
xL = [0;0;(1-epsilon)*L];

% Initial values

pN = mp0(1:3);
mN = mp0(4:6);
qN = q0;
 y(:,1) =[pN;mN;qN;x]; 
for j=1:n-1
         k_p(:,1)= h* getPdot(y(1:3,j),y(4:6,j),Cm);
         k_m(:,1)= h* getMdot(y(1:3,j),y(4:6,j),Cm,Cp);
         k_q(:,1)= h* getQdot(y(4:6,j),y(7:10,j),Cm);
         k_x(:,1)= h* getxdot(y(7:10,j),Cp,y(1:3,j));

         k_p(:,2)= h* getPdot(y(1:3,j) + .5*k_p(:,1),y(4:6,j) + .5*k_m(:,1),Cm);
         k_m(:,2)= h* getMdot(y(1:3,j) + .5*k_p(:,1),y(4:6,j) + .5*k_m(:,1),Cm,Cp);
         k_q(:,2)= h* getQdot(y(4:6,j) + .5*k_m(:,1),y(7:10,j) + .5*k_q(:,1),Cm);
         k_x(:,2)= h* getxdot(y(7:10,j) + .5*k_q(:,1),Cp,y(1:3,j) + .5*k_p(:,1));

         k_p(:,3)= h* getPdot(y(1:3,j) + .5*k_p(:,2),y(4:6,j)  + .5*k_m(:,2),Cm);
         k_m(:,3)= h* getMdot(y(1:3,j) + .5*k_p(:,2),y(4:6,j)  + .5*k_m(:,2),Cm,Cp);
         k_q(:,3)= h* getQdot(y(4:6,j) + .5*k_m(:,2),y(7:10,j)  + .5*k_q(:,2),Cm);
         k_x(:,3)= h* getxdot(y(7:10,j) + .5*k_q(:,2),Cp,y(1:3,j) + .5*k_p(:,2));

         k_p(:,4)= h* getPdot(y(1:3,j) + k_p(:,3),y(4:6,j) + k_m(:,3),Cm);
         k_m(:,4)= h* getMdot(y(1:3,j) + k_p(:,3),y(4:6,j) + k_m(:,3),Cm,Cp);
         k_q(:,4)= h* getQdot(y(4:6,j) + k_m(:,3),y(7:10,j) + k_q(:,3),Cm);
         k_x(:,4)= h* getxdot(y(7:10,j) + k_q(:,3),Cp,y(1:3,j) + k_p(:,3));

         y(1:3,j+1) = y(1:3,j) + (1/6)*(k_p * [1 ; 2 ; 2 ;1]);
         y(4:6,j+1) = y(4:6,j) + (1/6)*(k_m * [1 ; 2 ; 2 ;1]);
         y(7:10,j+1) = y(7:10,j) + (1/6)*(k_q * [1 ; 2 ; 2 ;1]);
          % renormalize Q
%              y(7:10,j+1) = y(7:10,j+1) ./ sqrt(dot(y(7:10,j+1),y(7:10,j+1)));   
         y(11:13,j+1) = y(11:13,j) + (1/6)*(k_x * [1 ; 2 ; 2 ;1]);
               
end

   F = [y(7:10,end);y(11:13,end)] - [qL;xL];
   v_is = v_i;
   nrm = norm(F);
        save('sol','y','v_is','nrm')

end

function qdot = getQdot(m,q,I)
    I = diag(I);
    invI = diag(1./I);
%Arnold Equation
qdot = .5*quatMult(q,([0;invI*m]));
qdot=qdot';
end
function pdot = getPdot(P,M,I)
Imat_inv = diag(diag(1./I));
pdot = -hat(Imat_inv*M)*P;                                     
end
function mdot = getMdot(P,M,I,E)
Imat = diag(diag(I));
Imat_inv = diag(diag(1./I));
Emat = diag(diag(E));
Emat_inv = diag(diag(1./E));
e3 = [0;0;1];
mdot = -hat(Imat_inv*M)*M - hat(Emat_inv*P)*P + hat(P)*e3;                                     
end

function xdot=getxdot(q,E,P)
Q =  QuatToMat(q);
Emat = diag(diag(E));
Emat_inv = diag(diag(1./E));
xdot = Q * Emat_inv * P + Q(:,3);
end

function vhat = hat(v);
vhat = [0, -v(3) , v(2);
        v(3) , 0 , -v(1);
        -v(2), v(1) , 0];
        
end
function exp_q = expQ(q)
        q0 = q(1);
        qv = q(2:end);
       exp_q = exp(q0).*[cos(norm(qv)) ;
                        qv./norm(qv)*sin(norm(qv))];
end
function [crossProd]=quatMult(q1,q2)
s1 = q1(1); v1 = q1(2:4);
s2 = q2(1); v2 = q2(2:4);
crossProd = [ s1*s2 - dot(v1,v2); s1*v2 + s2*v1 + cross(v1,v2)];
end
function [R]=QuatToMat(Quat)
R = [(1-2*(Quat(3)^2 + Quat(4)^2))         ,(-2*Quat(1)*Quat(4)+2*Quat(2)*Quat(3))  , (2*Quat(1)*Quat(3)+2*Quat(2)*Quat(4));
     (2*Quat(1)*Quat(4)+2*Quat(2)*Quat(3)) , (1 - 2*(Quat(2)^2 + Quat(4)^2))        , (-2*Quat(1)*Quat(2)+2*Quat(3)*Quat(4));
    (-2*Quat(1)*Quat(3)+2*Quat(2)*Quat(4)) , (2*Quat(1)*Quat(2)+2*Quat(3)*Quat(4))  , (1 - 2*(Quat(2)^2 + Quat(3)^2)) ];
end