%%%% PLEASE UNCOMMENT TESTCASES RUN
clear  
close all
%% Testcases 

%  testcase = 'Plot Convergence'
%  testcase = 'Plot Solution'
%  testcase = 'Plot Error in Preservation'
%  testcase = 'Effect of paramters on preservation' %This will take time

%% Initial Values
global N phi epsilon L Cm Cp; 
P0 = [.5;0;.5];
M0 = [cos(1.1);0;sin(1.1)];   
y0 = [M0;P0];
% Strain Measures
Cm = [110.8, 0, 0;
      0, 110.8, 0;
      0,0 , 85.2];
Cp = [2338.6, 0, 0;
      0, 2338.6, 0;
      0,0 , 6080.5];
  % Parameters for Bar Plots
ept = 0:0.2:0.8;
phipt = 0:pi/4:pi;
epTp = zeros(length(ept),length(phipt));
epTm = zeros(length(ept),length(phipt));
eH = zeros(length(ept),length(phipt));
tol = 1e-10; 
options = optimoptions('fsolve','Display','iter','Algorithm','levenberg-marquardt','PlotFcn',[],...
            'FunctionTolerance',tol); % fsolve options
switch testcase 
%% Plot Solution 
    case 'Plot Solution'
        phiVal = 0:pi/4:pi;
        epsilon = 0.2;
        L= 10;
        N = 512;
        figure
        for k=1:length(phiVal)
            phi = phiVal(k);
            [x_IM,F_IM] = fsolve(@Error_shootIM,y0,options);
            load('sol.mat')
            plot3(y(11,:),y(12,:),y(13,:),'-*','MarkerSize',5)
            hold on 
         end
        xlabel('x')
        ylabel('y')
        zlabel('z')
        set(gca, 'FontName', 'Times New Roman','FontSize',11)
        grid on
        box on
        legend('\phi =  0,      \xi = 0.4','\phi =  \pi/4,      \xi = 0.4','\phi =  \pi/2,      \xi = 0.4','\phi =  3\pi/2,      \xi = 0.4','\phi =  \pi,      \xi = .4')
%% Error in Presrevations
    case  'Plot Error in Preservation' 
            epsilon = 0.4;
            phi = pi;
            L= 10;
            N = 200;
            % EM  
            [x_EM,F_EM] = fsolve(@Error_shootEM,y0,options);
            acr_EM = norm(F_EM);
            load('sol.mat')
            y_EM = y;
            clear y
            % IM
            [x_IM,F_IM] = fsolve(@Error_shootIM,y0,options);
            acr_IM = norm(F_IM);
            load('sol.mat')
            y_IM = y;
            clear y
            % KM
            [x_KM,F_KM] = fsolve(@Error_shootKM,y0,options);
            acr_KM = norm(F_KM);
            load('sol.mat')
            y_KM = y;
            clear y
            % RK4
            [x_RK4,F_RK4] = fsolve(@Error_shootRK4,y0,options);
            acr_RK4 = norm(F_RK4);
            load('sol.mat')
            y_RK4 = y;
            clear y
            % GL4
            [x_GL4,F_GL4] = fsolve(@Error_shootGL4,y0,options);
            acr_GL4 = norm(F_GL4);
            load('sol.mat')
            y_GL4 = y;
            clear y
            [invariants_EM]=findInvariants(y_EM,Cm,Cp);
            [invariants_IM]=findInvariants(y_IM,Cm,Cp);
            [invariants_KM]=findInvariants(y_KM,Cm,Cp);
            [invariants_RK4]=findInvariants(y_RK4,Cm,Cp);
            [invariants_GL4]=findInvariants(y_GL4,Cm,Cp);
            %% ploting
            figure %% Invariant 1
            semilogy(max(eps,invariants_EM(1,:)), '-*','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM))
            hold on
            semilogy(max(eps,invariants_IM(1,:)), '-o','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            semilogy(max(eps,invariants_KM(1,:)), '-s','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            semilogy(max(eps,invariants_RK4(1,:)), '-d','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            semilogy(max(eps,invariants_GL4(1,:)), '-h','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            title('log(abs$( P_i^TP_i  -  P_0^TP_0 )$)','Interpreter','latex')
            xlabel('Grid points')
            ylabel('Error')
            legend('EM','IM','KM','RK4','GL4')
            set(gca, 'FontName', 'Times New Roman','FontSize',11)
            ylim([1e-15 1e5])
            xlim([0 N])
            grid on
            figure %% Invariant 2
            semilogy(max(eps,invariants_EM(2,:)), '-*','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            hold on
            semilogy(max(eps,invariants_IM(2,:)), '-o','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            semilogy(max(eps,invariants_KM(2,:)), '-s','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            semilogy(max(eps,invariants_RK4(2,:)), '-d','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            semilogy(max(eps,invariants_GL4(2,:)), '-h','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            title('log(abs$( P_i^TM_i  -  P_0^TM_0 )$)','Interpreter','latex')
            xlabel('Grid points')
            ylabel('Error')
            legend('EM','IM','KM','RK4','GL4')
            set(gca, 'FontName', 'Times New Roman','FontSize',11)
            ylim([1e-15 1e5])
            xlim([0 N])
            grid on
            figure %% Invariant 3
            semilogy(max(eps,invariants_EM(3,:)), '-*','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            hold on
            semilogy(max(eps,invariants_IM(3,:)), '-o','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            semilogy(max(eps,invariants_KM(3,:)), '-s','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            semilogy(max(eps,invariants_RK4(3,:)), '-d','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            semilogy(max(eps,invariants_GL4(3,:)), '-h','LineWidth',1.5,'MarkerSize',7,'MarkerIndices',1:5:length(y_IM)) 
            title('log(abs$( \mathcal{J}_i - \mathcal{J}_0 )$) ','Interpreter','latex')
            xlabel('Grid points')
            ylabel('Error')
            legend('EM','IM','KM','RK4','GL4')
            set(gca, 'FontName', 'Times New Roman','FontSize',11)
            ylim([1e-15 1e5])
            xlim([0 N])
            grid on
%% Effect of Paramters on Preservation
    case  'Effect of paramters on preservation'
            L= 10;
            N = 200;
            for i=1:length(ept)
                for j=1:length(phipt)
                    epsilon = ept(i);
                    phi = phipt(j);
                    % EM  
                    [x_EM,F_EM] = fsolve(@Error_shootEM,y0,options);
                    acr_EM = norm(F_EM);
                    load('sol.mat')
                    y_EM = y;
                    clear y
                    % IM
                    [x_IM,F_IM] = fsolve(@Error_shootIM,y0,options);
                    acr_IM = norm(F_IM);
                    load('sol.mat')
                    y_IM = y;
                    clear y
                    % KM
                    [x_KM,F_KM] = fsolve(@Error_shootKM,y0,options);
                    acr_KM = norm(F_KM);
                    load('sol.mat')
                    y_KM = y;
                    clear y
                    % RK4
                    [x_RK4,F_RK4] = fsolve(@Error_shootRK4,y0,options);
                    acr_RK4 = norm(F_RK4);
                    load('sol.mat')
                    y_RK4 = y;
                    clear y
                    % GL4
                    [x_GL4,F_GL4] = fsolve(@Error_shootGL4,y0,options);
                    acr_GL4 = norm(F_GL4);
                    load('sol.mat')
                    y_GL4 = y;
                    clear y
                    
                    %% Computing Error in Invariants
                    [invariants_EM]=findInvariants(y_EM,Cm,Cp);
                    [invariants_IM]=findInvariants(y_IM,Cm,Cp);
                    [invariants_KM]=findInvariants(y_KM,Cm,Cp);
                    [invariants_RK4]=findInvariants(y_RK4,Cm,Cp);
                    [invariants_GL4]=findInvariants(y_GL4,Cm,Cp);
                    
                    epTp_EM(i,j) = norm(abs(invariants_EM(1,:)),Inf);
                    epTm_EM(i,j) = norm(abs(invariants_EM(2,:)),Inf);
                    eH_EM(i,j) = norm(abs(invariants_EM(3,:)),Inf);
                    
                    epTp_IM(i,j) = norm(abs(invariants_IM(1,:)),Inf);
                    epTm_IM(i,j) = norm(abs(invariants_IM(2,:)),Inf);
                    eH_IM(i,j) = norm(abs(invariants_IM(3,:)),Inf);
                    
                    epTp_KM(i,j) = norm(abs(invariants_KM(1,:)),Inf);
                    epTm_KM(i,j) = norm(abs(invariants_KM(2,:)),Inf);
                    eH_KM(i,j) = norm(abs(invariants_KM(3,:)),Inf);
                    
                    epTp_RK4(i,j) = norm(abs(invariants_RK4(1,:)),Inf);
                    epTm_RK4(i,j) = norm(abs(invariants_RK4(2,:)),Inf);
                    eH_RK4(i,j) = norm(abs(invariants_RK4(3,:)),Inf);
                    
                    epTp_GL4(i,j) = norm(abs(invariants_GL4(1,:)),Inf);
                    epTm_GL4(i,j) = norm(abs(invariants_GL4(2,:)),Inf);
                    eH_GL4(i,j) = norm(abs(invariants_GL4(3,:)),Inf);
                end 
            end
            %% Plotting bar graph and Charts
            barplot(epTp_EM,epTm_EM,eH_EM,'EM')
            barplot(epTp_IM,epTm_IM,eH_IM,'IM')
            barplot(epTp_KM,epTm_KM,eH_KM,'KM')
            barplot(epTp_RK4,epTm_RK4,eH_RK4,'RK4')
            barplot(epTp_GL4,epTm_GL4,eH_GL4,'GL4')
%% Plot Convergence
    case 'Plot Convergence'        
        epsilon = 0.5;
        phi = pi;
        L= 1;
        load('ReferenceSolution.mat')
        N_h = 7;
        h_ref = L/(2^11);
        v_ref = 0:h_ref:L;
        for k = 4:N_h
             N = 2^k;
            % EM 
            tic
            [x_EM,F_EM] = fsolve(@Error_shootEM,y0,options);
            t_EM(k-3) = toc;
            acr_EM = norm(F_EM);
            load('sol.mat')
            y_EM = y;
            clear y
            % IM
            tic
            [x_IM,F_IM] = fsolve(@Error_shootIM,y0,options);
            t_IM(k-3) = toc;
            acr_IM = norm(F_IM);
            load('sol.mat')
            y_IM = y;
            clear y
            % KM
            tic
            [x_KM,F_KM] = fsolve(@Error_shootKM,y0,options);
            t_KM(k-3) = toc;
            acr_KM = norm(F_KM);
            load('sol.mat')
            y_KM = y;
            clear y
            % RK4
            tic
            [x_RK4,F_RK4] = fsolve(@Error_shootRK4,y0,options);
            t_RK4(k-3) = toc;
            acr_RK4 = norm(F_RK4);
            load('sol.mat')
            y_RK4 = y;
            clear y
            % GL4
            tic
            [x_GL4,F_GL4] = fsolve(@Error_shootGL4,y0,options);
            t_GL4(k-3) = toc;
            acr_GL4 = norm(F_GL4);
            load('sol.mat')
            y_GL4 = y;
            clear y
          
            h = L/(N);
            v_i = 0:h:L;
            a_i = find(ismember(v_i,v_ref));
            b_ref = find(ismember(v_ref,v_i));
            %%Computing Error
            for ept =1 : length(a_i)
                error_EM(k-3) = norm(y_ref(:,b_ref(ept)) - y_EM(:,a_i(ept)),Inf);
                error_IM(k-3) = norm(y_ref(:,b_ref(ept)) - y_IM(:,a_i(ept)),Inf);
                error_KM(k-3) = norm(y_ref(:,b_ref(ept)) - y_KM(:,a_i(ept)),Inf);
                error_RK4(k-3) = norm(y_ref(:,b_ref(ept)) - y_RK4(:,a_i(ept)),Inf);
                error_GL4(k-3)= norm(y_ref(:,b_ref(ept)) - y_GL4(:,a_i(ept)),Inf);
            end
            Scheme = {'EM';'IM';'KM';'RK4';'GL4'};
            fsolveAccuracy= [acr_EM;acr_IM;acr_KM;acr_RK4;acr_GL4];
            
        end
        % Error Plot
                    stp=4:N_h;
                    h = 1./2.^stp;
                    loglog(h,error_EM,'-*','LineWidth',1.5,'MarkerSize',10)
                    hold on
                    loglog(h,error_IM,'-o','LineWidth',1.5,'MarkerSize',10)
                    loglog(h,error_KM,'-s','LineWidth',1.5,'MarkerSize',10)
                    loglog(h,error_RK4,'-d','LineWidth',1.5,'MarkerSize',10)
                    loglog(h,error_GL4,'-p','LineWidth',1.5,'MarkerSize',10)
                    axis equal
                    title('Convergence Plots')
                    xlabel('Step-Size')
                    ylabel('Global Error')
                    box on
                    set(gca, 'FontName', 'Times New Roman','FontSize',11)
                    p_IM = polyfit(log10(h(2:end)),log10(error_EM(2:end)),1);
                    p_KM = polyfit(log10(h(2:end)),log10(error_IM(2:end)),1);
                    p_SM = polyfit(log10(h(2:end)),log10(error_KM(2:end)),1);
                    p_RK4 = polyfit(log10(h(2:end)),log10(error_RK4(2:end)),1);
                    p_GL4 = polyfit(log10(h(2:end)),log10(error_GL4(2:end)),1);
                    ConvergenceRate = [p_IM(1);p_KM(1);p_SM(1);p_RK4(1);p_GL4(1)];
                    T = table(Scheme,ConvergenceRate,fsolveAccuracy)
                    legend('EM','IM','KM','RK4','GL4')
                    
        % Time Plot
                    figure
                    loglog(h,t_EM,'-*','LineWidth',1.5,'MarkerSize',10)
                    hold on
                    loglog(h,t_IM,'-o','LineWidth',1.5,'MarkerSize',10)
                    loglog(h,t_KM,'-s','LineWidth',1.5,'MarkerSize',10)
                    loglog(h,t_RK4,'-d','LineWidth',1.5,'MarkerSize',10)
                    loglog(h,t_GL4,'-p','LineWidth',1.5,'MarkerSize',10)
                    xlabel('Step-size')
                    ylabel('Time')
                    legend('EM','IM','KM','RK4','GL4')
end
    
 %% Functions
function barplot(e1,e2,e3,method)
top = 1e-3;
bottom = 0;
% top  = max([max(max(e1)),max(max(e2)),max(max(e2))]);
% bottom = min([min(min(e1)),min(min(e2)),min(min(e3))]); 
colorlim = [bottom top];
%Invariant 1 bar
b1 = bar3(e1);
xticklabels({'0','\pi/4','\pi/2','3\pi/4','\pi'})
set(gca,'YTickLabel',[0 .2 .4 .6 .8],'FontName', 'Times New Roman','FontSize',11)
 xlabel('$\phi$','Interpreter','latex')
 ylabel('$\xi$','Interpreter','latex')
 zlabel('max(abs($ P_i^TP_i  -  P_0^TP_0$))','Interpreter','latex')
% zlabel('Error')
 for i = 1:length(b1)
    zdata1 = b1(i).ZData;
    b1(i).CData = zdata1;
    b1(i).FaceColor = 'interp';
 end
 caxis(colorlim)
 title(method)
 %invariant2 bar
figure
b2 = bar3(e2);
xticklabels({'0','\pi/4','\pi/2','3\pi/4','\pi'})
set(gca,'YTickLabel',[0 .2 .4 .6 .8],'FontName', 'Times New Roman','FontSize',11)
 xlabel('$\phi$','Interpreter','latex')
 ylabel('$\xi$','Interpreter','latex')
 zlabel('max(abs($ P_i^TM_i  -  P_0^TM_0$))','Interpreter','latex')
% zlabel('Error')
for j = 1:length(b2)
    zdata2 = b2(j).ZData;
    b2(j).CData = zdata2;
    b2(j).FaceColor = 'interp';
end
caxis(colorlim)
 
 title(method)
 %invariant3 bar
figure
b3 = bar3(e3);
xticklabels({'0','\pi/4','\pi/2','3\pi/4','\pi'})
set(gca,'YTickLabel',[0 .2 .4 .6 .8],'FontName', 'Times New Roman','FontSize',11)
xlabel('$\phi$','Interpreter','latex')
 ylabel('$\xi$','Interpreter','latex')
 zlabel('max(abs($ \mathcal{J}_i - \mathcal{J}_0 $))','Interpreter','latex')
% zlabel('Error')
for j = 1:length(b3)
    zdata2 = b3(j).ZData;
    b3(j).CData = zdata2;
    b3(j).FaceColor = 'interp';
end
caxis(colorlim)
title(method)
ept = 0:0.2:0.8;
phipt = 0:pi/4:pi;
% 
%invariant1 chart
figure
h1 = heatmap(phipt,ept,e1,'Colormap', parula(20),'ColorLimits',colorlim,'ColorbarVisible', 'off');
xlabel('\phi')
 ylabel('\xi')
title(['$P^TP$ using ', method])
set(gca,'FontName', 'Times New Roman','FontSize',11)
h1.NodeChildren(3).XAxis.TickLabelInterpreter = 'tex';
h1.XDisplayLabels = {'0', '\pi/4', '\pi/2', '3\pi/4', '\pi'};

%invariant2 chart
figure
h2 = heatmap(phipt,ept,e2,'Colormap', parula(20),'ColorLimits',colorlim,'ColorbarVisible', 'off');
xlabel('\phi')
 ylabel('\xi')
set(gca,'FontName', 'Times New Roman','FontSize',11)
h2.NodeChildren(3).XAxis.TickLabelInterpreter = 'tex';
h2.XDisplayLabels = {'0', '\pi/4', '\pi/2', '3\pi/4', '\pi'};
title(['$P^TM$ using ', method])
%invariant3 chart
figure
h3 = heatmap(phipt,ept,e3,'Colormap', parula(20),'ColorLimits',colorlim,'ColorbarVisible', 'off');
xlabel('\phi')
 ylabel('\xi')
set(gca,'FontName', 'Times New Roman','FontSize',11)
h3.NodeChildren(3).XAxis.TickLabelInterpreter = 'tex';
h3.XDisplayLabels = {'0', '\pi/4', '\pi/2', '3\pi/4', '\pi'};
title(['$\mathcal{J}$ using ', method])
end

function invar = findInvariants(y,Cm,Cp)
inv1 = abs(sum((y(1:3,:)').^2-(y(1:3,1)').^2,2));
inv2 = abs(sum((y(1:3,:)').*(y(4:6,:)')-(y(1:3,1)').*(y(4:6,1)'),2));
inv3 = abs(.5*sum((y(1:3,:)').*(Cp\y(1:3,:))',2) + .5*sum((y(4:6,:)').*(Cm\y(4:6,:))',2) +...
            sum((y(1:3,:)').*[0 0 1],2) + sum((y(7:10,:)').^2,2)- ...
            (.5*sum((y(1:3,1)').*(Cp\y(1:3,1))',2) + .5*sum((y(4:6,1)').*(Cm\y(4:6,1))',2) +...
            sum((y(1:3,1)').*[0 0 1],2) + sum((y(7:10,1)').^2,2)));
invar = [inv1';inv2';inv3'];

end